#!/bin/sh
#   Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of dSS11 WebSetup.
#
#   This file is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This file is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

export PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin
export SHELL=/bin/sh

. /www/pages/cgi-bin/errlib.sh
. /www/pages/cgi-bin/util.sh
. /www/pages/cgi-bin/webservice.sh

DSS_CONFIG_D="/usr/share/dss/data/config.d/webservice.xml"
KITE_SCRIPT="/www/pages/cgi-bin/pagekiteconfig.sh"
MODE="$1"
USERNAME=
PASSWORD=
DATASHARING=
VALIDITY_HOURS=

if [[ "$MODE" != "temp" ]] && [[ "$MODE" != "persistent" ]]; then
    log "Error: invalid mode parameter \"$MODE\", must be temp or persistent"
    exit 1
fi

if [[ "$MODE" == "persistent" ]]; then
    USERNAME=$(urlencode "$2")
    PASSWORD=$(urlencode "$3")
    DATASHARING="$4"
    if [[ -z "$USERNAME" ]] || [[ -z "$PASSWORD" ]] || [[ -z "$DATASHARING" ]]; then
        log "Error: at least one of username/password/datasharing parameters is missing"
        exit 1
    fi
else
    KITE_SCRIPT="/www/pages/cgi-bin/tempkiteconfig.sh"
    VALIDITY_HOURS="$2"
    if [[ -z "$VALIDITY_HOURS" ]]; then
        log "Error: validity parameter is missing"
        exit 1
    fi
fi

log "Webservice authentication started."

log "Querying RemoteConnectivity tokens..."
DSS_JSON_RESPONSE=$(curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/property/query?query=%2Fsystem%2Fsecurity%2FapplicationTokens%2F*%2F*(token%2CapplicationName")
RETCODE=$?
if [[ $RETCODE -ne 0 ]]; then
    log "Error: could not send request to query tokens from dSS, aborting, curl returned [$RETCODE]"
    exit 1
fi

SUCCESS=$(echo "$DSS_JSON_RESPONSE" | jq -M -r .ok)
if [[ "$SUCCESS" != "true" ]]; then
    log "Error: could not query application tokens, dSS replied with failure"
    exit 1
fi

TOKEN=
for tkn in $(echo "$DSS_JSON_RESPONSE" | jq  -M -r '.result.enabled[] | select(.applicationName == "RemoteConnectivity") | .token')
do
    log "Found token $tkn"
    if [[ "$MODE" != "persistent" ]]; then
        if [[ -n "$tkn" ]]; then
            log "Setting token $tkn"
            TOKEN="$tkn"
            break
        fi
    else
        curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/system/revokeToken?applicationToken=$tkn" 2>&1 >/dev/null
   fi
done

DEFAULT=$(echo -n "dssadmin:dSS11:dssadmin" | md5sum | cut -b -32)
DEFAULT="dssadmin:dSS11:$DEFAULT"
CURRENT=$(cat /etc/lighttpd.user  | grep dssadmin | head -n 1)

if [[ "x$DEFAULT" == "x$CURRENT" ]]; then
    log "Error: user has default password set, not enabling remote connectivity"
    curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/event/raise?name=remote-connectivity&parameter=success%3Dfalse%3Bmode%3D$MODE%3Bcode%3D$ERR_DEFAULT_PW%3Bservicecode%3D0" 2>&1 >/dev/null
    exit 1
fi

if [[ "$MODE" == "persistent" ]]; then
    log "Requesting new application token for RemoteConnectivity"

    DSS_JSON_RESPONSE=$(curl -s "http://127.0.0.1:8088/json/system/requestApplicationToken?applicationName=RemoteConnectivity")
    RETCODE=$?
    if [[ $RETCODE -ne 0 ]]; then
        log "Error: could not request token from dSS, aborting, curl returned [$RETCODE]"
        exit 1
    fi

    SUCCESS=$(echo "$DSS_JSON_RESPONSE" | jq -M -r .ok)
    if [[ "$SUCCESS" != "true" ]]; then
        log "Error: could not request application token, dSS replied with failure"
        exit 1
    fi

    TOKEN=$(echo "$DSS_JSON_RESPONSE" | jq -M -r .result.applicationToken)

    if [[ -z "$TOKEN" ]]; then
        log "Error: received empty token from dSS"
        exit 1
    fi

    DSS_JSON_RESPONSE=$(curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/system/enableToken?applicationToken=$TOKEN")
    RETCODE=$?
    if [[ $RETCODE -ne 0 ]]; then
        log "Error: could not ask dSS to enable token, curl returned [$RETCODE]"
        exit 1
    fi

    SUCCESS=$(echo "$DSS_JSON_RESPONSE" | jq -M -r .ok)
    if [[ "$SUCCESS" != "true" ]]; then
        log "Error: could not enable application token, dSS replied with failure"
        exit 1
    fi

    log "Enabled token $TOKEN on dSS"
else
    if [[ -z "$TOKEN" ]]; then
        log "Could not retrieve old application token from dSS, aborting installer link setup!"
        exit 1
    fi
    log "Retrieved token $TOKEN from dSS"
fi

dss_handle_error() {
    if [[ "$MODE" == "persistent" ]]; then
        log "Disabling kite..."
        sh /www/pages/cgi-bin/pagekiteconfig.sh off 2>&1 >/dev/null
        log "Revoking token on dSS..."
        curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/system/revokeToken?applicationToken=$TOKEN" 2>&1 >/dev/null
    else
        log "Disabling tempkite..."
        sh /www/pages/cgi-bin/tempkiteconfig.sh off 2>&1 >/dev/null
    fi
    curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/event/raise?name=remote-connectivity&parameter=success%3Dfalse%3Bmode%3D$MODE%3Bcode%3D$1%3Bservicecode%3D$2" 2>&1 >/dev/null
}

DSID=`mac2dsid -i eth0`
MAC=`ifconfig eth0 | grep 'HWaddr' | awk '{ print $5}'`
UUID=`uuidgen | sed s/-//g`
NAME_PAGEKITE="$UUID"
RELAY_LINK="$NAME_PAGEKITE.digitalstrom.net"

RETCODE=1
if [[ "$MODE" == "persistent" ]]; then
    log "request ActivateRemoteConnectivity for MAC-address $MAC, user $USERNAME"
    WEBSERVICE_RESPONSE=$(curl -s -X POST -H "Content-Length: 0" "$WEBSERVICE_BASEURL/internal/accessmanagement/v1_0/RemoteConnectivity/ActivateRemoteConnectivity?relayLink=${RELAY_LINK}&dssid=${DSID}&dsssgtin=${DSID}&macAddress=${MAC}&token=${TOKEN}&user=${USERNAME}&password=${PASSWORD}&dataSharingMode=${DATASHARING}")
    RETCODE=$?
else
    log "request ActivateTempdSConfiguratorLink for DSID $DSID"
    WEBSERVICE_RESPONSE=$(curl -s -X POST -H "Content-Length: 0" "$WEBSERVICE_BASEURL/internal/accessmanagement/v1_0/RemoteConnectivity/ActivateTempdSConfiguratorLink?relayLink=${RELAY_LINK}&dssid=${DSID}&dsssgtin=${DSID}&token=${TOKEN}&tempLinkValidity=${VALIDITY_HOURS}&type=Installer")
    RETCODE=$?
fi

if [[ $RETCODE -ne 0 ]]; then
    log "Error: could not send request to webservice, curl returned [$RETCODE]"
    dss_handle_error $ERR_WEB_SERVICE
    exit 1
fi

SUCCESS=$(echo "$WEBSERVICE_RESPONSE" | jq -M -r .ReturnCode)
if [[ -z "$SUCCESS" ]] || [[ "$SUCCESS" != "0" ]]; then
    ERR_MSG=$(echo "$WEBSERVICE_RESPONSE" | jq -M -r .ReturnMessage)
    log "Error: webservice returned $SUCCESS and said: $ERR_MSG"
    dss_handle_error $ERR_WEB_SERVICE $SUCCESS
    exit 1
fi

RELAY_PASSWORD=$(echo "$WEBSERVICE_RESPONSE" | jq -M -r .Response.RelayPassword)
if [[ -z "$RELAY_PASSWORD" ]]; then
    log "Error: webservice did not return a valid relay password!"
    dss_handle_error $ERR_INVALID $SUCCESS
    exit 1
fi

log "Activating pagekite in $MODE mode"

KITE_MSG=$(sh $KITE_SCRIPT configure "$NAME_PAGEKITE" "$RELAY_PASSWORD")
RETCODE="$?"
if [[ $RETCODE -ne 0 ]]; then
    log "Erorr configuring pagekite: $KITE_MSG"
    dss_handle_error $ERR_KITE_CONFIG
    exit 1
fi

if [[ "$MODE" == "persistent" ]]; then
    KITE_MSG=$(sh $KITE_SCRIPT on)
    RETCODE="$?"
else
    KITE_MSG=$(sh $KITE_SCRIPT on $VALIDITY_HOURS)
    RETCODE="$?"
fi

if [[ $RETCODE -ne 0 ]]; then
    log "Erorr enabling pagekite: $KITE_MSG"
    dss_handle_error $ERR_KITE_ENABLE
    exit 1
fi

if [[ "$MODE" == "persistent" ]]; then
    # write config.d
    cat > $DSS_CONFIG_D << EOL
<?xml version="1.0" encoding="utf-8"?>
<properties version="1">
  <property name="config">
    <property name="webservice-api/enabled" type="boolean">
      <value>true</value>
    </property>
  </property>
</properties>
EOL

    DSS_JSON_RESPONSE=$(curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/property/setBoolean?path=/config/webservice-api/enabled&value=true")
    RETCODE=$?
    if [[ $RETCODE -ne 0 ]]; then
        log "Error: could not set webservice api property on dSS, curl returned [$RETCODE]"
        exit 1
    fi

    SUCCESS=$(echo "$DSS_JSON_RESPONSE" | jq -M -r .ok)
    if [[ "$SUCCESS" != "true" ]]; then
        log "Error: could not set webservice api property on dSS"
        exit 1
    fi
fi

DSS_JSON_RESPONSE=$(curl -s -H "Authorization: $HTTP_AUTHORIZATION" -H "Cookie: $HTTP_COOKIE" "http://127.0.0.1:8088/json/event/raise?name=remote-connectivity&parameter=success%3D${SUCCESS}%3Bosptoken%3D${TOKEN}%3Brelaylink%3D${RELAY_LINK}%3Bmode%3D${MODE}%3Bcode%3D${ERR_OK}")
RETCODE=$?
if [[ $RETCODE -ne 0 ]]; then
    log "Error: could not raise event on dSS, curl returned [$RETCODE]"
    exit 1
fi

SUCCESS=$(echo "$DSS_JSON_RESPONSE" | jq -M -r .ok)
if [[ "$SUCCESS" != "true" ]]; then
    log "Error: could not notify dSS about webservice authentication"
    exit 1
fi

log "Webservice authentication done."
